-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio                   "EskaQuestTracker.Quests"                           ""
-- ========================================================================== --
namespace "EQT"
-- ========================================================================== --
GetNumQuestLogEntries      = GetNumQuestLogEntries
GetQuestLogTitle           = GetQuestLogTitle
GetNumQuestLogEntries      = GetNumQuestLogEntries
GetQuestLogTitle           = GetQuestLogTitle
GetQuestLogIndexByID       = GetQuestLogIndexByID
GetQuestWatchIndex         = GetQuestWatchIndex
GetQuestLogSpecialItemInfo = GetQuestLogSpecialItemInfo
GetQuestObjectiveInfo      = GetQuestObjectiveInfo
GetDistanceSqToQuest       = GetDistanceSqToQuest
AddQuestWatch              = AddQuestWatch
SelectQuestLogEntry        = SelectQuestLogEntry
IsWorldQuest               = QuestUtils_IsQuestWorldQuest
IsQuestBounty              = IsQuestBounty
-- IsQuestTask
-- ======================[[ DATA ]]========================================== --
QUESTS_CACHE               = {}
QUEST_HEADERS_CACHE        = {}
QUEST_LOCATIONS            = {}
-- ========================================================================== --
function OnLoad(self)
  _DB:SetDefault("Quests", {
    sortByDistance = true,
    filteringByZone = false,
  })

end

__Thread__()
function OnEnable(self)
  if not _QuestBlock then
    _QuestBlock = QuestBlock()
    _Addon:RegisterBlock(_QuestBlock)
     Wait("QUEST_LOG_UPDATE")
     --self:ShowPopup(14456, "COMPLETE")
  end

  Debug("Quest module is enabled")

  self:LoadQuests()
  self:UpdateQuestHeadersCache()
  self:UpdateDistance()
  self:RefreshPopup()
end


function OnDisable(self)
  if _QuestBlock then
    _QuestBlock.isActive = false
  end

end

__SystemEvent__()
function PLAYER_ALIVE()
  _M:LoadQuests()
end

__Thread__()
__SystemEvent__()
function QUEST_ACCEPTED(index, questID)
  -- Don't continue if the quest is a world quest
  if IsWorldQuest(questID) or IsQuestBounty(questID) then return end

  Debug("Accept  quest id :  %i", questID)

  -- Get a quest object
  local quest = _ObjectManager:GetQuest()
  quest.id = questID
  quest.header = _M:GetQuestHeader(questID)
  quest.level = select(2, GetQuestLogTitle(GetQuestLogIndexByID(questID)))

  -- Add in the cache
  QUESTS_CACHE[quest.id] = quest

  -- @HACK : Set a little delay to get a valid quest item
  Delay(0.1)
  -- Update the quest and get its objectives
  _M:UpdateQuest(quest)

  -- Add it in the block
  _QuestBlock:AddQuest(quest)
  -- Refresh the block displaying
  _QuestBlock:Draw()
end

__SystemEvent__()
function QUEST_REMOVED(questID)
  _QuestBlock:RemoveQuest(questID)
  _QuestBlock:Draw()
end


function LoadQuests(self)
  local numEntries, numQuests = GetNumQuestLogEntries()
  local currentHeader = "MISC"

  for i = 1, numEntries do
    local title, level, suggestedGroup, isHeader, isCollapsed, isComplete,
    frequency, questID, startEvent, displayQuestID, isOnMap, hasLocalPOI,
    isTask, isBounty, isStory, isHidden = GetQuestLogTitle(i)

    if not isTask and not QUESTS_CACHE[questID] then
      if isHeader then
        currentHeader = title
      elseif not isHeader and not isHidden then
        AddQuestWatch(i)
        local quest = _ObjectManager:GetQuest()
        quest.id = questID
        quest.name = title
        quest.level =  level
        quest.isHidden = isHidden
        quest.header = currentHeader

        QUESTS_CACHE[questID] = quest
        self:UpdateQuest(quest)

        _QuestBlock:AddQuest(quest)
      end
    end
  end
  _QuestBlock:Draw()
end

function UpdateQuestHeadersCache()
  wipe(QUEST_HEADERS_CACHE)
  local currentHeader = "Misc"
  local numEntries, numQuests = GetNumQuestLogEntries()

  for i = 1, numEntries do
      local title, _, _, isHeader, _, _, _, questID = GetQuestLogTitle(i)
      if isHeader then
        currentHeader = title
      else
        QUEST_HEADERS_CACHE[questID] = currentHeader
      end
  end
end

function GetQuestHeader(self, qID)
  -- Check if the quest header is in the cache
  if QUEST_HEADERS_CACHE[qID] then
    -- if yes, return the header in the cache
    return QUEST_HEADERS_CACHE[qID]
  end

  -- if no, find the quest header
  local currentHeader = "Misc"
  local numEntries, numQuests = GetNumQuestLogEntries()

  for i = 1, numEntries do
    local title, _, _, isHeader, _, _, _, questID = GetQuestLogTitle(i)
    if isHeader then
      currentHeader = title
    elseif questID == qID then
      QUEST_HEADERS_CACHE[qID] = currentHeader
      return currentHeader
    end
  end

  return currentHeader
end

function UpdateQuest(self, quest)
  local questLogIndex = GetQuestLogIndexByID(quest.id)
  local questWatchIndex = GetQuestWatchIndex(questLogIndex)

  local distanceSq, onContinent = GetDistanceSqToQuest(questLogIndex)

  if not questWatchIndex then return end

    local questID, title, questLogIndex, numObjectives, requiredMoney,
    isComplete, startEvent, isAutoComplete, failureTime, timeElapsed,
    questType, isTask, isBounty, isStory, isOnMap, hasLocalPOI = GetQuestWatchInfo(questWatchIndex)

    local itemLink, itemTexture = GetQuestLogSpecialItemInfo(questLogIndex)

    quest.name = title
    quest.isOnMap = isOnMap
    quest.isTask = isTask
    quest.isBounty = isBounty
    quest.distance = distanceSq

    if itemLink and itemTexture then
      local itemQuest = quest:GetQuestItem()

      itemQuest.link = itemLink
      itemQuest.texture = itemTexture
    end

    if numObjectives > 0 then
      quest.numObjectives = numObjectives
      for index = 1, numObjectives do
        local text, type, finished = GetQuestObjectiveInfo(quest.id, index, false)
        local objective = quest:GetObjective(index)

        objective.isCompleted = finished
        objective.text = text

        if type == "progressbar" then
          local progress = GetQuestProgressBarPercent(quest.id)
          objective:ShowProgress()
          objective:SetMinMaxProgress(0, 100)
          objective:SetProgress(progress)
          objective:SetTextProgress(string.format("%i%%", progress))
        else
          objective:HideProgress()
        end

      end
    else
      quest.numObjectives = 1
      local objective = quest:GetObjective(1)
      SelectQuestLogEntry(questLogIndex)
      objective.text = GetQuestLogCompletionText()
      objective.isCompleted = false
    end
end

__SystemEvent__ "PLAYER_ENTERING_WORLD" "QUEST_LOG_UPDATE" "QUEST_WATCH_LIST_CHANGED" "QUEST_POI_UPDATE"
"ZONE_CHANGED_NEW_AREA" "ZONE_CHANGED"
function QUESTS_UPDATE()
  for _, quest in pairs(QUESTS_CACHE) do
    _M:UpdateQuest(quest)
  end
end


__Thread__()
function UpdateDistance()
  while QuestBlock.sortByDistance do
    for _, quest in pairs(QUESTS_CACHE) do
      local questLogIndex = GetQuestLogIndexByID(quest.id)
      local distanceSq, onContinent = GetDistanceSqToQuest(questLogIndex)

      quest.distance = math.sqrt(distanceSq)
    end
    _QuestBlock:Draw()
    Next()
  end
end

-- ========================================================================== --
-- == Quest Popup
-- ========================================================================== --
-- @TODO : Maybe move this class in an another file ?
class "QuestPopup" extend "IFrame"

  local function UpdateType(self, new)
    if new == "OFFER" then
      self.frame.topName:SetText(QUEST_WATCH_POPUP_QUEST_DISCOVERED:upper())
      self.frame.topName:SetTextColor(1, 106 / 255, 0)
      self.frame.icon:SetTexCoord(0.13476563, 0.17187500, 0.01562500, 0.53125000)

      self.frame.subName:SetText(QUEST_WATCH_POPUP_CLICK_TO_VIEW)
    elseif new == "COMPLETE" then
      self.frame.topName:SetText("QUEST COMPLETE")
      self.frame.topName:SetTextColor(0, 220/255, 0)
      self.frame.icon:SetTexCoord(0.17578125, 0.21289063, 0.01562500, 0.53125000)

      self.frame.subName:SetText(QUEST_WATCH_POPUP_CLICK_TO_COMPLETE)
    end
  end

  local function UpdateName(self, new)
    self.frame.name:SetText(new)
  end

  property "name" { TYPE = String, DEFAULT = "", HANDLER = UpdateName }
  property "type" { TYPE = String, DEFAULT = "", HANDLER = UpdateType }

  function QuestPopup(self)
    local frame = CreateFrame("Frame")
    frame:SetHeight(32)
    frame:SetBackdrop(_Backdrops.CommonWithBiggerBorder)
    --frame:SetBackdropColor(1.0, 216/255, 0, 0.5)
    frame:SetBackdropColor(127/255, 0, 0, 0.5)
    frame:SetBackdropBorderColor(0, 0, 0)

    local icon = frame:CreateTexture(nil, "OVERLAY")
    icon:SetSize(19 * 0.85, 33 * 0.85)
    icon:SetPoint("TOPLEFT", 10, -2)
    icon:SetTexture([[Interface\QuestFrame\AutoQuest-Parts]])
    icon:SetTexCoord(0.13476563, 0.17187500, 0.01562500, 0.53125000)
    frame.icon = icon

    local stripe = frame:CreateTexture(nil, "ARTWORK")
    stripe:SetPoint("CENTER", icon, "CENTER")
    stripe:SetSize(32 * 0.85, 32 * 0.85)
    stripe:SetTexture([[Interface\AddOns\EskaQuestTracker\Media\Textures\Stripe]])
    stripe:SetVertexColor(0.25, 0.25, 0.25)
    icon.stripe = stripe

    local bgStripe = frame:CreateTexture(nil, "BORDER")
    bgStripe:SetAllPoints(stripe)
    bgStripe:SetColorTexture(0, 0, 0)
    icon.bg = bgStripe

    local name = frame:CreateFontString(nil, "OVERLAY")
    local fontName = _LibSharedMedia:Fetch("font", "PT Sans Bold")
    local fontSub = _LibSharedMedia:Fetch("font", "PT Sans Narrow Bold")

    name:SetPoint("LEFT", stripe, "RIGHT")
    name:SetPoint("RIGHT")
    name:SetJustifyH("CENTER")
    name:SetFont(fontName, 10)
    name:SetText("")
    --name:SetTextColor(1, 106 / 255, 0)
    name:SetTextColor(1, 216/255, 0)
    frame.name = name

    local topName = frame:CreateFontString(nil, "OVERLAY")
    topName:SetPoint("TOPLEFT", stripe, "TOPRIGHT")
    topName:SetPoint("BOTTOM", name, "TOP")
    topName:SetPoint("RIGHT")
    topName:SetJustifyH("CENTER")
    topName:SetFont(fontSub, 8)
    topName:SetText("QUEST DISCOVERED")
    topName:SetTextColor(0, 220/255, 0)
    --topName:SetTextColor(1, 106 / 255, 0)
    frame.topName = topName


    local subName = frame:CreateFontString(nil, "OVERLAY")
    subName:SetPoint("BOTTOMLEFT", stripe, "BOTTOMRIGHT")
    subName:SetPoint("RIGHT")
    subName:SetPoint("TOP", name, "BOTTOM")
    subName:SetJustifyH("CENTER")
    subName:SetFont(fontSub, 7)
    subName:SetText("Click to complete the quest")
    frame.subName = subName

    frame:SetScript("OnEnter", function(self)
      self:SetBackdropColor(180/255, 0, 0, 0.75)
    end)

    frame:SetScript("OnLeave", function(self)
      self:SetBackdropColor(127/255, 0, 0, 0.5)
    end)

    self.frame = frame

  end
endclass "QuestPopup"


function ShowPopup(self, questID, popupType)
  if not _M.popup then
    _M.popup = QuestPopup()
    _M.popup.frame:SetPoint("BOTTOMLEFT", _Addon.ObjectiveTracker.frame, "TOPLEFT")
    _M.popup.frame:SetPoint("BOTTOMRIGHT", _Addon.ObjectiveTracker.frame, "TOPRIGHT")
  else
    _M.popup:Show()
  end

  _M.popup.type = popupType
  _M.popup.name = GetQuestLogTitle(GetQuestLogIndexByID(questID))

  if popupType == "OFFER" then
    _M.popup.frame:SetScript("OnMouseDown", function()
      ShowQuestOffer(GetQuestLogIndexByID(questID))
      RemoveAutoQuestPopUp(questID)
      _M.popup:Hide()
    end)
  else
    _M.popup.frame:SetScript("OnMouseDown", function()
      ShowQuestComplete(GetQuestLogIndexByID(questID))
      RemoveAutoQuestPopUp(questID)
      _M.popup:Hide()
    end)
  end

end

function RefreshPopup(self)
  for i = 1, GetNumAutoQuestPopUps() do
		local questID, popUpType = GetAutoQuestPopUp(i)
    if ( not IsQuestBounty(questID) ) then
      self:ShowPopup(questID, popUpType)
    end
  end
end


__SecureHook__()
function AutoQuestPopupTracker_AddPopUp(questID, popupType)
  Wait(function() _M:ShowPopup(questID, popupType) end, "QUEST_WATCH_LIST_CHANGED", "QUEST_LOG_UPDATE")
end

__SecureHook__()
function AutoQuestPopupTracker_RemovePopUp(questID, popupType)
  if _M.popup then
    _M.popup:Hide()
  end
end

__SystemEvent__()
function QUEST_AUTOCOMPLETE(...)
  local questID = ...
  _M:ShowPopup(questID, "COMPLETE")
end
